require 'yaml'

module Redk::CapnPanda
  class Config

    def self.config_file
      ENV['CAPN_PANDA_CONFIG'] || ENV['HOME'] + "/.config/capn_panda.yaml"
    end

    def self.load
      unless File.exist?(Config.config_file)
        self.save(Config.config_file, Config.default_config)
      end

      self.new(Config.config_file)
    end

    def self.nuke
      File.delete(Config.config_file) if File.exist?(Config.config_file)
    end

    def self.save(config_file, config)
      File.open(Config.config_file, 'w+') {|f| f.write(config.to_yaml) }
    end

    def self.default_config
      {
        jira_domain: "jira.atlassian.com",
        jira_username: "jsmith",
        jira_default_project_key: "PROJ",
        jira_default_issuetype_name: "Task",
        jira_gerrit_field: "customfield_10403",
        jira_new_issue_fields: {
          # customfield_12700: {
          #   value: "platform"
          # }
        },

        gerrit_domain: "gerrit.changeme.com",
        firework_domain: false
      }
    end

    def initialize(config_file)
      @default_config = Config.default_config
      @config = @default_config.merge(::YAML::load(File.open(config_file)))
    end

    def write
      Config.save(Config.config_file, @config)
    end

    def set(key, value)
      @config[key.to_sym] = value
    end

    def to_ary
      return @config.values
    end

    def to_hash
      return @config
    end

    def method_missing(m, *args, &block)
      return @config[m.to_sym] if @config.has_key?(m.to_sym)
      raise "no such config property: #{m}"
    end
  end
end
