module Redk::CapnPanda
  class JiraClient < JIRA::Client

    def self.jira_domain
      Redk::CapnPanda.config.jira_domain
    end

    def inspect
      "<Redk::CapnPanda::Jira::Client site: #{options[:site]}>"
    end

    def my_issues
      @my_issues ||= (
        my_issues_jql = "assignee = currentUser() AND resolution = Unresolved ORDER BY updatedDate DESC"
        issues = JIRA::Resource::Issue.jql(self, my_issues_jql)
        issues
      )
    end

    def self.from_keychain
      @keychain_client ||= (
        keychain_item = Keychain.default.internet_passwords.where(server: self.jira_domain).first
        raise Redk::CapnPanda::KeyChainError, "#{self.jira_domain} is in your keychain" unless keychain_item
        self.from_keychain_item(keychain_item) if keychain_item
      )
    end

    def self.from_keychain_item(keychain_item)
      options = {
                  :username => keychain_item.account,
                  :password => keychain_item.password,
                  :site     => "https://#{self.jira_domain}",
                  :context_path => '',
                  :auth_type => :basic
                }
      self.new(options)
    end
  end
end

JIRA::Resource::Issue.class_eval do

  def has_branch?
    @has_branch ||= branch != nil
  end

  def branch
    @branch ||= Redk::CapnPanda.git.branches.local.select do |git_branch|
      git_branch.name[/#{self.key}$/]
    end.first
  end

  def checkout(type='issue')
    Redk::CapnPanda.git.branch("#{type}/#{self.key}").checkout
  end

  def move_to_in_progress
    transition = self.transitions.build
    id = transitions.all.first {|e| puts e.name == "To In Progress" }.id
    transition.save!("transition" => {"id" => id})
  end
end
